@@ -1,5 +1,3 @@
-from celery import chain
-
 from django.contrib import admin
 from django.db import models
 from django.db.models.functions import Lower
@@ -12,21 +10,21 @@ from . import __title__
 from .app_settings import STRUCTURES_DEVELOPER_MODE
 from .models import (
     EveCategory,
-    EveGroup,
-    EveType,
-    EveRegion,
     EveConstellation,
-    EveSolarSystem,
+    EveEntity,
+    EveGroup,
     EveMoon,
     EvePlanet,
+    EveRegion,
+    EveSolarSystem,
     EveSovereigntyMap,
+    EveType,
+    Notification,
+    Owner,
+    Structure,
     StructureTag,
     StructureService,
     Webhook,
-    EveEntity,
-    Owner,
-    Notification,
-    Structure,
 )
 from . import tasks
 from .utils import LoggerAddTag
@@ -116,14 +114,18 @@ class NotificationAdmin(admin.ModelAdmin):
         if names:
             return ", ".join(names)
         else:
-            return None
+            return format_html(
+                '<span style="color: red"></i>Error: This notification can not be sent, '
+                "because there is no webhook configured for {}. ",
+                obj.owner,
+            )
 
     _webhooks.short_description = "Webhooks"
 
     actions = (
         "mark_as_sent",
         "mark_as_unsent",
-        "send_to_webhook",
+        "send_to_webhooks",
         "process_for_timerboard",
     )
 
@@ -153,7 +155,7 @@ class NotificationAdmin(admin.ModelAdmin):
 
     mark_as_unsent.short_description = "Mark selected notifications as unsent"
 
-    def send_to_webhook(self, request, queryset):
+    def send_to_webhooks(self, request, queryset):
         obj_pks = [obj.pk for obj in queryset]
         tasks.send_notifications.delay(obj_pks)
 
@@ -163,7 +165,7 @@ class NotificationAdmin(admin.ModelAdmin):
             "configured webhooks".format(len(obj_pks)),
         )
 
-    send_to_webhook.short_description = (
+    send_to_webhooks.short_description = (
         "Send selected notifications to configured webhooks"
     )
 
@@ -228,18 +230,22 @@ class OwnerAdmin(admin.ModelAdmin):
         "_corporation",
         "_alliance",
         "character",
+        "_is_active",
         "_webhooks",
         "has_default_pings_enabled",
         "_ping_groups",
-        "_is_active",
+        "_is_alliance_main",
         "_is_structure_sync_ok",
         "_is_notification_sync_ok",
         "_is_forwarding_sync_ok",
+        "_structures_count",
+        "_notifications_count",
     )
     list_filter = (
         ("corporation__alliance", admin.RelatedOnlyFieldListFilter),
         "has_default_pings_enabled",
         "is_active",
+        "is_alliance_main",
         OwnerSyncStatusFilter,
     )
     ordering = ["corporation__corporation_name"]
@@ -269,7 +275,10 @@ class OwnerAdmin(admin.ModelAdmin):
         if names:
             return names
         else:
-            return None
+            return format_html(
+                '<span style="color: red"></i>Error: Notifications can not be sent, '
+                "because there is no webhook configured for this owner."
+            )
 
     def _is_active(self, obj):
         return obj.is_active
@@ -277,6 +286,12 @@ class OwnerAdmin(admin.ModelAdmin):
     _is_active.boolean = True
     _is_active.short_description = "active"
 
+    def _is_alliance_main(self, obj):
+        return obj.is_alliance_main
+
+    _is_alliance_main.boolean = True
+    _is_alliance_main.short_description = "alliance main"
+
     def _is_structure_sync_ok(self, obj):
         if not obj.is_active:
             return None
@@ -304,6 +319,58 @@ class OwnerAdmin(admin.ModelAdmin):
     _is_forwarding_sync_ok.boolean = True
     _is_forwarding_sync_ok.short_description = "forwarding"
 
+    def _notifications_count(self, obj: Owner) -> int:
+        return obj.notification_set.count()
+
+    _notifications_count.short_description = "notifications"
+
+    def _structures_count(self, obj: Owner) -> int:
+        return obj.structure_set.count()
+
+    _structures_count.short_description = "structures"
+
+    actions = (
+        "update_structures",
+        "fetch_notifications",
+        "deactivate_owners",
+        "activate_owners",
+    )
+
+    def activate_owners(self, request, queryset):
+        queryset.update(is_active=True)
+        self.message_user(request, f"Activated {queryset.count()} owners")
+
+    activate_owners.short_description = "Activate selected owners"
+
+    def deactivate_owners(self, request, queryset):
+        queryset.update(is_active=False)
+        self.message_user(request, f"Deactivated {queryset.count()} owners")
+
+    deactivate_owners.short_description = "Deactivate selected owners"
+
+    def update_structures(self, request, queryset):
+        for obj in queryset:
+            tasks.update_structures_for_owner.delay(obj.pk, user_pk=request.user.pk)
+            text = "Started updating structures for: {}. ".format(obj)
+            text += "You will receive a notification once it is completed."
+
+            self.message_user(request, text)
+
+    update_structures.short_description = "Update structures from EVE server"
+
+    def fetch_notifications(self, request, queryset):
+        for obj in queryset:
+            tasks.process_notifications_for_owner.delay(obj.pk, user_pk=request.user.pk)
+            text = "Started fetching notifications for: {}. ".format(obj)
+            text += "You will receive a notification once it is completed."
+
+            self.message_user(request, text)
+
+    fetch_notifications.short_description = "Fetch notifications from EVE server"
+
+    def has_add_permission(self, request):
+        return True if STRUCTURES_DEVELOPER_MODE else False
+
     def get_readonly_fields(self, request, obj=None):
         if obj:  # editing an existing object
             return self.readonly_fields + (
@@ -355,44 +422,6 @@ class OwnerAdmin(admin.ModelAdmin):
         ),
     )
 
-    actions = ("update_structures", "fetch_notifications", "send_notifications")
-
-    def update_structures(self, request, queryset):
-        for obj in queryset:
-            tasks.update_structures_for_owner.delay(obj.pk, user_pk=request.user.pk)
-            text = "Started updating structures for: {}. ".format(obj)
-            text += "You will receive a notification once it is completed."
-
-            self.message_user(request, text)
-
-    update_structures.short_description = "Update structures from EVE server"
-
-    def fetch_notifications(self, request, queryset):
-        for obj in queryset:
-            tasks.fetch_notifications_for_owner.delay(obj.pk, user_pk=request.user.pk)
-            text = "Started fetching notifications for: {}. ".format(obj)
-            text += "You will receive a notification once it is completed."
-
-            self.message_user(request, text)
-
-    fetch_notifications.short_description = "Fetch notifications from EVE server"
-
-    def send_notifications(self, request, queryset):
-        send_tasks = list()
-        for owner in queryset:
-            send_tasks.append(
-                tasks.send_new_notifications_for_owner.si(owner_pk=owner.pk)
-            )
-            self.message_user(
-                request, "Started sending new notifications for: {}. ".format(owner)
-            )
-        chain(send_tasks).delay()
-
-    send_notifications.short_description = "Send new notifications to Discord"
-
-    def has_add_permission(self, request):
-        return True if STRUCTURES_DEVELOPER_MODE else False
-
 
 @admin.register(StructureTag)
 class StructureTagAdmin(admin.ModelAdmin):
@@ -668,6 +697,7 @@ class WebhookAdmin(admin.ModelAdmin):
         "_ping_groups",
         "is_active",
         "is_default",
+        "_messages_in_queue",
     )
     list_filter = ("webhook_type", "has_default_pings_enabled", "is_active")
     save_as = True
@@ -679,7 +709,10 @@ class WebhookAdmin(admin.ModelAdmin):
         else:
             return None
 
-    actions = ("test_notification", "activate", "deactivate")
+    def _messages_in_queue(self, obj):
+        return obj.queue_size()
+
+    actions = ("test_notification", "activate", "deactivate", "purge_messages")
 
     def test_notification(self, request, queryset):
         for obj in queryset:
@@ -712,4 +745,19 @@ class WebhookAdmin(admin.ModelAdmin):
 
     deactivate.short_description = "Deactivate selected profiles"
 
+    def purge_messages(self, request, queryset):
+        actions_count = 0
+        killmails_deleted = 0
+        for webhook in queryset:
+            killmails_deleted += webhook.clear_queue()
+            actions_count += 1
+
+        self.message_user(
+            request,
+            f"Purged queued messages for {actions_count} webhooks, "
+            f"deleting a total of {killmails_deleted} messages.",
+        )
+
+    purge_messages.short_description = "Purge queued messages from selected webhooks"
+
     filter_horizontal = ("ping_groups",)
