@@ -1,11 +1,9 @@
 """Notification related models"""
 
 from datetime import datetime, timedelta
-import json
 import logging
-from time import sleep
 import yaml
-from typing import List, Set
+from typing import List, Set, Tuple
 
 import dhooks_lite
 import pytz
@@ -15,8 +13,8 @@ from django.db import models
 from django.contrib.auth.models import Group
 from django.core.validators import MinValueValidator
 from django.conf import settings
-from django.utils.translation import gettext_lazy as _, gettext
 from django.utils import translation
+from django.utils.translation import gettext_lazy as _, gettext
 
 from allianceauth.eveonline.evelinks import dotlan
 
@@ -27,13 +25,13 @@ from ..app_settings import (
     STRUCTURES_DEFAULT_LANGUAGE,
     STRUCTURES_DEVELOPER_MODE,
     STRUCTURES_MOON_EXTRACTION_TIMERS_ENABLED,
-    STRUCTURES_NOTIFICATION_MAX_RETRIES,
-    STRUCTURES_NOTIFICATION_WAIT_SEC,
     STRUCTURES_NOTIFICATION_SHOW_MOON_ORE,
+    STRUCTURES_NOTIFICATION_SET_AVATAR,
     STRUCTURES_REPORT_NPC_ATTACKS,
     STRUCTURES_TIMERS_ARE_CORP_RESTRICTED,
 )
 from .. import __title__
+from ..webhooks.models import WebhookBase
 from ..managers import EveEntityManager
 from ..utils import (
     app_labels,
@@ -165,35 +163,9 @@ def get_default_notification_types():
     return tuple(sorted([str(x[0]) for x in NTYPE_CHOICES]))
 
 
-class Webhook(models.Model):
+class Webhook(WebhookBase):
     """A destination for forwarding notification alerts"""
 
-    TYPE_DISCORD = 1
-
-    TYPE_CHOICES = [
-        (TYPE_DISCORD, _("Discord Webhook")),
-    ]
-
-    name = models.CharField(
-        max_length=64, unique=True, help_text="short name to identify this webhook"
-    )
-    webhook_type = models.IntegerField(
-        choices=TYPE_CHOICES, default=TYPE_DISCORD, help_text="type of this webhook"
-    )
-    url = models.CharField(
-        max_length=255,
-        unique=True,
-        help_text=(
-            "URL of this webhook, e.g. "
-            "https://discordapp.com/api/webhooks/123456/abcdef"
-        ),
-    )
-    notes = models.TextField(
-        null=True,
-        default=None,
-        blank=True,
-        help_text="you can add notes about this webhook here if you want",
-    )
     notification_types = MultiSelectField(
         choices=NTYPE_CHOICES,
         default=get_default_notification_types,
@@ -208,10 +180,6 @@ class Webhook(models.Model):
         verbose_name="language",
         help_text="language of notifications send to this webhook",
     )
-    is_active = models.BooleanField(
-        default=True,
-        help_text="whether notifications are currently sent to this webhook",
-    )
     is_default = models.BooleanField(
         default=False,
         help_text=(
@@ -232,31 +200,6 @@ class Webhook(models.Model):
         help_text="Groups to be pinged for each notification - ",
     )
 
-    def __str__(self) -> str:
-        return self.name
-
-    def __repr__(self) -> str:
-        return "{}(id={}, name='{}')".format(
-            self.__class__.__name__, self.id, self.name
-        )
-
-    def send_test_notification(self) -> str:
-        """Sends a test notification to this webhook and returns send report"""
-        hook = dhooks_lite.Webhook(self.url)
-        response = hook.execute(
-            _(
-                "This is a test notification from %s.\n"
-                "The webhook appears to be correctly configured."
-            )
-            % __title__,
-            wait_for_response=True,
-        )
-        if response.status_ok:
-            send_report_json = json.dumps(response.content, indent=4, sort_keys=True)
-        else:
-            send_report_json = "HTTP status code {}".format(response.status_code)
-        return send_report_json
-
 
 class EveEntity(models.Model):
     """An EVE entity like a character or an alliance"""
@@ -456,17 +399,7 @@ class Notification(models.Model):
         """
         add_prefix = make_logger_prefix("notification:{}".format(self.notification_id))
         logger.info(add_prefix("Trying to sent to webhook: %s" % webhook))
-        if self.is_alliance_level:
-            avatar_url = self.owner.corporation.alliance.logo_url()
-            ticker = self.owner.corporation.alliance.alliance_ticker
-        else:
-            avatar_url = self.owner.corporation.logo_url()
-            ticker = self.owner.corporation.corporation_ticker
 
-        username = gettext("%(ticker)s Notification") % {"ticker": ticker}
-        hook = dhooks_lite.Webhook(
-            webhook.url, username=username, avatar_url=avatar_url
-        )
         success = False
         try:
             embed = self._generate_embed(webhook.language_code)
@@ -503,9 +436,35 @@ class Notification(models.Model):
                             if role:
                                 content += f" <@&{role['id']}>"
 
-            success = self._execute_webhook(hook, content, embed, add_prefix)
+            username, avatar_url = self._gen_avatar()
+            success = webhook.send_message(
+                content=content,
+                embeds=[embed],
+                username=username,
+                avatar_url=avatar_url,
+            )
+            if success:
+                self.is_sent = True
+                self.save()
+
         return success
 
+    def _gen_avatar(self) -> Tuple[str, str]:
+        if STRUCTURES_NOTIFICATION_SET_AVATAR:
+            if self.is_alliance_level:
+                avatar_url = self.owner.corporation.alliance.logo_url()
+                ticker = self.owner.corporation.alliance.alliance_ticker
+            else:
+                avatar_url = self.owner.corporation.logo_url()
+                ticker = self.owner.corporation.corporation_ticker
+
+            username = gettext("%(ticker)s Notification") % {"ticker": ticker}
+        else:
+            username = None
+            avatar_url = None
+
+        return username, avatar_url
+
     @staticmethod
     def _import_discord() -> object:
         from allianceauth.services.modules.discord.models import DiscordUser
@@ -601,10 +560,9 @@ class Notification(models.Model):
     def _gen_embed_structures_1(self, parsed_text: dict) -> dhooks_lite.Embed:
 
         try:
-            my_structure = Structure.objects.get(id=parsed_text["structureID"])
-            structure_name = my_structure.name
-            structure_type = my_structure.eve_type
-            structure_solar_system = my_structure.eve_solar_system
+            my_structure = Structure.objects.select_related().get(
+                id=parsed_text["structureID"]
+            )
         except Structure.DoesNotExist:
             my_structure = None
             structure_name = gettext("(unknown)")
@@ -614,13 +572,21 @@ class Notification(models.Model):
             structure_solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
                 parsed_text["solarsystemID"]
             )
+            owner_link = "(unknown)"
+        else:
+            structure_name = my_structure.name
+            structure_type = my_structure.eve_type
+            structure_solar_system = my_structure.eve_solar_system
+            owner_link = self._gen_corporation_link(str(my_structure.owner))
 
         description = gettext(
             "The %(structure_type)s %(structure_name)s in %(solar_system)s "
+            "belonging to %(owner_link)s "
         ) % {
             "structure_type": structure_type.name_localized,
             "structure_name": "**%s**" % structure_name,
             "solar_system": self._gen_solar_system_text(structure_solar_system),
+            "owner_link": owner_link,
         }
         if self.notification_type == NTYPE_STRUCTURE_ONLINE:
             title = gettext("Structure online")
@@ -741,10 +707,15 @@ class Notification(models.Model):
             solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
                 parsed_text["solarsystemID"]
             )
+            owner_link = self._gen_corporation_link(
+                parsed_text.get("ownerCorpName", "(unknown)")
+            )
             description = gettext(
-                "%(structure_type)s has started anchoring " "in %(solar_system)s. "
+                "A %(structure_type)s belonging to %(owner_link)s "
+                "has started anchoring in %(solar_system)s. "
             ) % {
                 "structure_type": structure_type.name_localized,
+                "owner_link": owner_link,
                 "solar_system": self._gen_solar_system_text(solar_system),
             }
             if not solar_system.is_null_sec:
@@ -789,6 +760,7 @@ class Notification(models.Model):
         )
         solar_system_link = self._gen_solar_system_text(solar_system)
         structure_name = parsed_text["structureName"]
+        owner_link = self._gen_corporation_link(str(self.owner))
         if self.notification_type == NTYPE_MOONS_EXTRACTION_STARTED:
             started_by, _ = EveEntity.objects.get_or_create_esi(
                 parsed_text["startedBy"]
@@ -798,7 +770,8 @@ class Notification(models.Model):
             title = gettext("Moon mining extraction started")
             description = gettext(
                 "A moon mining extraction has been started "
-                "for %(structure_name)s at %(moon)s in %(solar_system)s. "
+                "for %(structure_name)s at %(moon)s in %(solar_system)s "
+                "belonging to %(owner_link)s. "
                 "Extraction was started by %(character)s.\n"
                 "The chunk will be ready on location at %(ready_time)s, "
                 "and will autofracture on %(auto_time)s.\n"
@@ -807,6 +780,7 @@ class Notification(models.Model):
                 "structure_name": "**%s**" % structure_name,
                 "moon": moon.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "character": started_by,
                 "ready_time": ready_time.strftime(DATETIME_FORMAT),
                 "auto_time": auto_time.strftime(DATETIME_FORMAT),
@@ -824,7 +798,8 @@ class Notification(models.Model):
             title = gettext("Extraction finished")
             description = gettext(
                 "The extraction for %(structure_name)s at %(moon)s "
-                "in %(solar_system)s is finished and the chunk is ready "
+                "in %(solar_system)s belonging to %(owner_link)s "
+                "is finished and the chunk is ready "
                 "to be shot at.\n"
                 "The chunk will automatically fracture on %(auto_time)s.\n"
                 "%(ore_text)s"
@@ -832,6 +807,7 @@ class Notification(models.Model):
                 "structure_name": "**%s**" % structure_name,
                 "moon": moon.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "auto_time": auto_time.strftime(DATETIME_FORMAT),
                 "ore_text": gettext(
                     "\nOre composition: %s" % self._ore_composition_text(parsed_text)
@@ -845,13 +821,15 @@ class Notification(models.Model):
             title = gettext("Automatic Fracture")
             description = gettext(
                 "The moondrill fitted to %(structure_name)s at %(moon)s"
-                " in %(solar_system)s has automatically been fired "
+                " in %(solar_system)s belonging to %(owner_link)s "
+                "has automatically been fired "
                 "and the moon products are ready to be harvested.\n"
                 "%(ore_text)s"
             ) % {
                 "structure_name": "**%s**" % structure_name,
                 "moon": moon.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "ore_text": gettext(
                     "\nOre composition: %s" % self._ore_composition_text(parsed_text)
                 )
@@ -870,11 +848,13 @@ class Notification(models.Model):
             title = gettext("Extraction cancelled")
             description = gettext(
                 "An ongoing extraction for %(structure_name)s at %(moon)s "
-                "in %(solar_system)s has been cancelled by %(character)s."
+                "in %(solar_system)s belonging to %(owner_link)s "
+                "has been cancelled by %(character)s."
             ) % {
                 "structure_name": "**%s**" % structure_name,
                 "moon": moon.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "character": cancelled_by,
             }
 
@@ -885,13 +865,15 @@ class Notification(models.Model):
             title = gettext("Moondrill fired")
             description = gettext(
                 "The moondrill fitted to %(structure_name)s at %(moon)s "
-                "in %(solar_system)s has been fired by %(character)s "
+                "in %(solar_system)s belonging to %(owner_link)s "
+                "has been fired by %(character)s "
                 "and the moon products are ready to be harvested.\n"
                 "%(ore_text)s"
             ) % {
                 "structure_name": "**%s**" % structure_name,
                 "moon": moon.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "character": fired_by,
                 "ore_text": gettext(
                     "\nOre composition: %s" % self._ore_composition_text(parsed_text)
@@ -921,17 +903,20 @@ class Notification(models.Model):
             parsed_text["solarSystemID"]
         )
         solar_system_link = self._gen_solar_system_text(solar_system)
+        owner_link = self._gen_corporation_link(str(self.owner))
         aggressor_link = self._get_aggressor_link(parsed_text)
 
         if self.notification_type == NTYPE_ORBITAL_ATTACKED:
             title = gettext("Orbital under attack")
             description = gettext(
                 "The %(structure_type)s at %(planet)s in %(solar_system)s "
+                "belonging to %(owner_link)s "
                 "is under attack by %(aggressor)s."
             ) % {
                 "structure_type": structure_type.name_localized,
                 "planet": planet.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "aggressor": aggressor_link,
             }
             color = self.EMBED_COLOR_WARNING
@@ -943,12 +928,14 @@ class Notification(models.Model):
             title = gettext("Orbital reinforced")
             description = gettext(
                 "The %(structure_type)s at %(planet)s in %(solar_system)s "
+                "belonging to %(owner_link)s "
                 "has been reinforced by %(aggressor)s "
                 "and will come out at: %(date)s."
             ) % {
                 "structure_type": structure_type.name_localized,
                 "planet": planet.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "aggressor": aggressor_link,
                 "date": reinforce_exit_time.strftime(DATETIME_FORMAT),
             }
@@ -968,6 +955,7 @@ class Notification(models.Model):
         eve_moon, _ = EveMoon.objects.get_or_create_esi(parsed_text["moonID"])
         structure_type, _ = EveType.objects.get_or_create_esi(parsed_text["typeID"])
         solar_system_link = self._gen_solar_system_text(eve_moon.eve_solar_system)
+        owner_link = self._gen_corporation_link(str(self.owner))
         qs_structures = Structure.objects.filter(eve_moon=eve_moon)
         if qs_structures.exists():
             structure_name = qs_structures.first().name
@@ -992,12 +980,14 @@ class Notification(models.Model):
             title = gettext("Starbase under attack")
             description = gettext(
                 "The starbase %(structure_name)s at %(moon)s "
-                "in %(solar_system)s is under attack by %(aggressor)s.\n"
+                "in %(solar_system)s belonging to %(owner_link)s "
+                "is under attack by %(aggressor)s.\n"
                 "%(damage_text)s"
             ) % {
                 "structure_name": "**%s**" % structure_name,
                 "moon": eve_moon.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "aggressor": aggressor_link,
                 "damage_text": damage_text,
             }
@@ -1008,12 +998,13 @@ class Notification(models.Model):
             title = gettext("Starbase low on fuel")
             description = gettext(
                 "The starbase %(structure_name)s at %(moon)s "
-                "in %(solar_system)s is low on fuel. "
+                "in %(solar_system)s belonging to %(owner_link)s is low on fuel. "
                 "It has %(quantity)d fuel blocks left."
             ) % {
                 "structure_name": "**%s**" % structure_name,
                 "moon": eve_moon.name_localized,
                 "solar_system": solar_system_link,
+                "owner_link": owner_link,
                 "quantity": quantity,
             }
             color = self.EMBED_COLOR_WARNING
@@ -1045,9 +1036,7 @@ class Notification(models.Model):
 
         structure_type, _ = EveType.objects.get_or_create_esi(structure_type_id)
         structure_type_name = structure_type.name_localized
-        sov_owner_link = self._gen_alliance_link(
-            self.owner.corporation.alliance.alliance_name
-        )
+        sov_owner_link = self._gen_alliance_link(self.sender.name)
         if self.notification_type == NTYPE_SOV_ENTOSIS_CAPTURE_STARTED:
             title = gettext(
                 "%(structure_type)s in %(solar_system)s is being captured"
@@ -1076,11 +1065,12 @@ class Notification(models.Model):
             }
             description = gettext(
                 "Command nodes for %(structure_type)s in %(solar_system)s "
-                "can now be found throughout "
+                "belonging to %(owner)s can now be found throughout "
                 "the %(constellation)s constellation"
             ) % {
                 "structure_type": "**%s**" % structure_type_name,
                 "solar_system": solar_system_link,
+                "owner": sov_owner_link,
                 "constellation": solar_system.eve_constellation.name_localized,
             }
             color = self.EMBED_COLOR_WARNING
@@ -1155,20 +1145,21 @@ class Notification(models.Model):
 
     @classmethod
     def _gen_solar_system_text(cls, solar_system: EveSolarSystem) -> str:
-        text = "[{}]({}) ({})".format(
-            solar_system.name_localized,
-            dotlan.solar_system_url(solar_system.name),
+        text = "{} ({})".format(
+            Webhook.create_link(
+                solar_system.name_localized, dotlan.solar_system_url(solar_system.name)
+            ),
             solar_system.eve_constellation.eve_region.name_localized,
         )
         return text
 
     @classmethod
-    def _gen_alliance_link(cls, alliance_name: str):
-        return "[{}]({})".format(alliance_name, dotlan.alliance_url(alliance_name))
+    def _gen_alliance_link(cls, alliance_name: str) -> str:
+        return Webhook.create_link(alliance_name, dotlan.alliance_url(alliance_name))
 
     @classmethod
-    def _gen_corporation_link(cls, corporation_name: str):
-        return "[{}]({})".format(
+    def _gen_corporation_link(cls, corporation_name: str) -> str:
+        return Webhook.create_link(
             corporation_name, dotlan.corporation_url(corporation_name)
         )
 
@@ -1177,9 +1168,9 @@ class Notification(models.Model):
         """returns the attacker link from a parsed_text
         For Upwell structures only
         """
-        if "allianceName" in parsed_text:
+        if parsed_text.get("allianceName"):
             name = cls._gen_alliance_link(parsed_text["allianceName"])
-        elif "corpName" in parsed_text:
+        elif parsed_text.get("corpName"):
             name = cls._gen_corporation_link(parsed_text["corpName"])
         else:
             name = "(unknown)"
@@ -1191,17 +1182,17 @@ class Notification(models.Model):
         """returns the aggressor link from a parsed_text
         for POS and POCOs only
         """
-        if "aggressorAllianceID" in parsed_text:
+        if parsed_text.get("aggressorAllianceID"):
             key = "aggressorAllianceID"
-        elif "aggressorCorpID" in parsed_text:
+        elif parsed_text.get("aggressorCorpID"):
             key = "aggressorCorpID"
-        elif "aggressorID" in parsed_text:
+        elif parsed_text.get("aggressorID"):
             key = "aggressorID"
         else:
             return "(Unknown aggressor)"
 
         entity, _ = EveEntity.objects.get_or_create_esi(parsed_text[key])
-        return "[{}]({})".format(entity.name, entity.profile_url())
+        return Webhook.create_link(entity.name, entity.profile_url())
 
     @classmethod
     def _get_type_id_from_event_type(cls, event_type: int) -> int:
@@ -1210,57 +1201,6 @@ class Notification(models.Model):
         else:
             return None
 
-    def _execute_webhook(self, hook, content, embed, add_prefix) -> bool:
-        """executes webhook for sending the message, will retry on errors
-
-        Sets this notification as "sent" if successful
-
-        returns True/False on success
-        """
-        success = False
-        max_retries = STRUCTURES_NOTIFICATION_MAX_RETRIES
-        for retry_count in range(max_retries + 1):
-            if retry_count > 0:
-                logger.warn(
-                    add_prefix("Retry {} / {}".format(retry_count, max_retries))
-                )
-            try:
-                res = hook.execute(
-                    content=content, embeds=[embed], wait_for_response=True
-                )
-                if res.status_ok:
-                    self.is_sent = True
-                    self.save()
-                    success = True
-                    break
-
-                elif res.status_code == self.HTTP_CODE_TOO_MANY_REQUESTS:
-                    if "retry_after" in res.content:
-                        retry_after = res.content["retry_after"] / 1000
-                    else:
-                        retry_after = STRUCTURES_NOTIFICATION_WAIT_SEC
-                    logger.warn(
-                        add_prefix(
-                            "rate limited - will retry after %d secs" % retry_after
-                        )
-                    )
-                    sleep(retry_after)
-
-                else:
-                    logger.warn(add_prefix("Failed to send message"))
-                    break
-
-            except Exception as ex:
-                logger.warn(
-                    add_prefix("Unexpected issue when trying to send message: %s" % ex)
-                )
-                if settings.DEBUG:
-                    raise ex
-                else:
-                    break
-
-        return success
-
     def process_for_timerboard(self, token: Token = None) -> bool:
         """add/removes a timer related to this notification for some types
         returns True when a timer was processed, else False
@@ -1374,10 +1314,7 @@ class Notification(models.Model):
                 visibility=visibility,
                 structure_name=structure_obj.name,
                 owner_name=self.owner.corporation.corporation_name,
-                details_notes=(
-                    "Automatically created from structure notification for "
-                    f"{self.owner.corporation}"
-                ),
+                details_notes=self._timer_details_notes(),
             )
             timer_added = True
 
@@ -1429,10 +1366,7 @@ class Notification(models.Model):
                     eve_alliance=self.owner.corporation.alliance,
                     visibility=visibility,
                     owner_name=self.owner.corporation.corporation_name,
-                    details_notes=(
-                        "Automatically created from structure notification for "
-                        f"{self.owner.corporation}"
-                    ),
+                    details_notes=self._timer_details_notes(),
                 )
                 timer_added = True
 
@@ -1490,11 +1424,8 @@ class Notification(models.Model):
                 eve_corporation=self.owner.corporation,
                 eve_alliance=self.owner.corporation.alliance,
                 visibility=visibility,
-                owner_name=self.owner.corporation.corporation_name,
-                details_notes=(
-                    "Automatically created from structure notification for "
-                    f"{self.owner.corporation}"
-                ),
+                owner_name=self.sender.name,
+                details_notes=self._timer_details_notes(),
             )
             timer_added = True
 
@@ -1545,10 +1476,7 @@ class Notification(models.Model):
                 visibility=visibility,
                 structure_name="Customs Office",
                 owner_name=self.owner.corporation.corporation_name,
-                details_notes=(
-                    "Automatically created from structure notification for "
-                    f"{self.owner.corporation}"
-                ),
+                details_notes=self._timer_details_notes(),
             )
             timer_added = True
 
@@ -1626,10 +1554,7 @@ class Notification(models.Model):
                     visibility=visibility,
                     structure_name=parsed_text["structureName"],
                     owner_name=self.owner.corporation.corporation_name,
-                    details_notes=(
-                        "Automatically created from structure notification for "
-                        f"{self.owner.corporation}"
-                    ),
+                    details_notes=self._timer_details_notes(),
                 )
                 timer_added = True
 
@@ -1681,3 +1606,10 @@ class Notification(models.Model):
                         )
 
         return timer_added
+
+    def _timer_details_notes(self) -> str:
+        """returns generated details notes string for Timers"""
+        return (
+            "Automatically created from structure notification for "
+            f"{self.owner.corporation} at {self.timestamp.strftime(DATETIME_FORMAT)}"
+        )
